/* ----------------------------------------------------------------------------
 * Copyright (c) 2017 Semiconductor Components Industries, LLC (d/b/a
 * ON Semiconductor), All Rights Reserved
 *
 * This code is the property of ON Semiconductor and may not be redistributed
 * in any form without prior written permission from ON Semiconductor.
 * The terms of use and warranty for this code are covered by contractual
 * agreements between ON Semiconductor and the licensee.
 *
 * This is Reusable Code.
 *
 * ------------------------------------------------------------------------- */

#include <malloc.h>
#include <string.h>

#include "scratch.h"

#include "ioG722Encode.h"

static OperationParameters parameters;

static uint8_t oneFrame[80] =
{
    0xb7, 0x29, 0x84, 0x20, 0x84, 0x20, 0xba, 0xbf, 0xbe, 0xbc, 0x3e, 0x3f, 0x3d,
    0x3d, 0xbd, 0xbd, 0xbf, 0xbf, 0x3f, 0x3f, 0x3d, 0x3d, 0xbd, 0xbd, 0xbf,
    0xbf, 0x3f, 0x3f, 0x3d, 0x3d, 0xbd, 0xbd, 0xbf, 0xbf, 0x3f, 0x3f, 0x3d,
    0x3d, 0xbd, 0xbd, 0xbf, 0xbf, 0x3f, 0x3f, 0x3d, 0x3d, 0xbd, 0xbd, 0xbf,
    0xbf, 0x3f, 0x3f, 0x3d, 0x3d, 0xbd, 0xbd, 0xbf, 0xbe, 0x3e, 0x3f, 0x3d,
    0x3d, 0xbc, 0xbd, 0xbf, 0xbf, 0x3f, 0x3f, 0x3f, 0x7c, 0xfc, 0xbd, 0xbf,
    0xbe, 0x1f, 0x3a, 0x3b, 0x7b, 0xfa, 0xbc
};

/* Define function prototypes for the features we will override from the base */
static void g722Read(pSourceSink source, unsigned char *buffer,
                     uint32_t maxsize);

static void g722Write(pSourceSink sink, unsigned char *buffer, uint32_t size);

static int offset = 0;
static int counter = 0;

/* ----------------------------------------------------------------------------
 * Function     : makeG722DecodeSourceSink
 * ----------------------------------------------------------------------------
 * Description  : This IO provider allows test data to be input and output
 *                to the codec and is provided for debug purposes. This
 *                method allows the construction of a suitable control block
 *                using malloc.
 * Inputs       : None
 * Outputs      : Pointer to an IO Source/Sink control block which has been
 *                configured to match the test data indicated above.
 * Assumptions  : None
 * ------------------------------------------------------------------------- */
pSourceSink makeG722DecodeSourceSink(void)
{
    pSourceSink object = (pSourceSink)malloc(sizeof(struct _SourceSink));
    if (object != NULL)
    {
        object->read = &g722Read;
        object->write = &g722Write;
    }

    parameters.action = DECODE | DECODE_RESET;
    parameters.frameSize = 40;
    parameters.blockSize = 4;
    parameters.modeAndChannel = 1;
    offset = 0;

    return object;
}

/* ----------------------------------------------------------------------------
 * Function     : g722Read
 * ----------------------------------------------------------------------------
 * Description  : Implementation of the read method to provide data for test
 * Inputs       : source : pointer to a source/sink control block
 *                buffer : the buffer which will be used to return the test
 *                data
 *                maxsize : The maximum size of the buffer which must not be
 *                exceeded
 * Outputs      : None
 * Assumptions  : None
 * ------------------------------------------------------------------------- */
static void g722Read(pSourceSink source, unsigned char *buffer,
                     uint32_t maxsize)
{
    memcpy(buffer, &oneFrame[offset * parameters.frameSize],
           parameters.frameSize);
    memcpy(&source->operation, &parameters, sizeof(parameters));
    source->srcUsed = parameters.frameSize;
    parameters.action = DECODE;
    if (counter++ == 10)
    {
        parameters.action |= DECODE_PLC;
        counter = 0;
        scratchLog(0x111111);
    }
    offset = 1 - offset;
}

/* ----------------------------------------------------------------------------
 * Function     : g722Write
 * ----------------------------------------------------------------------------
 * Description  : Implementation of the write method to verify test results
 * Inputs       : source : pointer to a source/sink control block
 *                buffer : the buffer which has been used to provide the
 *                test results
 *                size : The used size of the buffer
 * Outputs      : None
 * Assumptions  : None
 * ------------------------------------------------------------------------- */
static void g722Write(pSourceSink sink, unsigned char *buffer, uint32_t size)
{
    /* don't need checks here, using debugger to verify correct behaviour */
}
