# ----------------------------------------------------------------------------
# Copyright (c) 2017 Semiconductor Components Industries, LLC (d/b/a
# ON Semiconductor), All Rights Reserved
#
# This code is the property of ON Semiconductor and may not be redistributed
# in any form without prior written permission from ON Semiconductor.
# The terms of use and warranty for this code are covered by contractual
# agreements between ON Semiconductor and the licensee.
#
# This is Reusable Code.
#
# -----------------------------------------------------------------------------
#

from threading  import Thread
from Queue import Queue, Empty

class PipeReader(Thread):
    '''
    Defines a pipereader class which allows us to easily communicate with
    subprocesses spawned using POpen
    '''
    
    def __init__(self, pipe):
        '''
        Constructor takes a pipe to read from and creates a queue to hold the
        data being read 
        '''
        super(PipeReader, self).__init__()
        self.pipe = pipe
        self.queue = Queue()
        self.daemon = True
        
    def run(self):
        '''
        Run method will constantly read the next line from the pipe and store
        it in the queue 
        '''
        for line in iter(self.pipe.readline, b''):
            self.queue.put(line)
        self.pipe.close()

    def readLine(self):
        '''
        This method will try to return the next line in the queue if one is
        available.
        '''
        try:
            line = self.queue.get_nowait()
        except Empty:
            return None
        else:
            return line.rstrip("\n")
        
    def canRead(self):
        '''
        indicate ifthe pipe can be read from
        '''
        return self.is_alive() and not self.queue.empty()
    
